--
-- Type: TABLE; Owner: I2B2DEMODATA; Name: OBSERVATION_FACT
--
 CREATE TABLE "I2B2DEMODATA"."OBSERVATION_FACT"
  (	"ENCOUNTER_NUM" NUMBER(38,0) NOT NULL ENABLE,
"PATIENT_NUM" NUMBER(38,0) NOT NULL ENABLE,
"CONCEPT_CD" VARCHAR2(50 BYTE) NOT NULL ENABLE,
"PROVIDER_ID" VARCHAR2(50 BYTE) NOT NULL ENABLE,
"START_DATE" DATE NOT NULL,
"MODIFIER_CD" VARCHAR2(100 BYTE) DEFAULT '@' NOT NULL ENABLE,
"INSTANCE_NUM" NUMBER(18,0) DEFAULT '1' NOT NULL ENABLE,
"VALTYPE_CD" VARCHAR2(50 BYTE),
"TVAL_CHAR" VARCHAR2(255 BYTE),
"NVAL_NUM" NUMBER(18,5),
"VALUEFLAG_CD" VARCHAR2(50 BYTE),
"QUANTITY_NUM" NUMBER(18,5),
"UNITS_CD" VARCHAR2(50 BYTE),
"END_DATE" DATE,
"LOCATION_CD" VARCHAR2(50 BYTE),
"OBSERVATION_BLOB" CLOB,
"CONFIDENCE_NUM" NUMBER(18,5),
"UPDATE_DATE" DATE,
"DOWNLOAD_DATE" DATE,
"IMPORT_DATE" DATE,
"SOURCESYSTEM_CD" VARCHAR2(50 BYTE),
"UPLOAD_ID" NUMBER(38,0),
 CONSTRAINT "OBSERVATION_FACT_PK" PRIMARY KEY ("PATIENT_NUM", "CONCEPT_CD", "MODIFIER_CD", "START_DATE", "ENCOUNTER_NUM", "INSTANCE_NUM", "PROVIDER_ID")
 USING INDEX
 TABLESPACE "I2B2_INDEX"  ENABLE
  )
 TABLESPACE "I2B2"
LOB ("OBSERVATION_BLOB") STORE AS SECUREFILE (
 TABLESPACE "I2B2" ENABLE STORAGE IN ROW CHUNK 8192
 NOCACHE NOLOGGING  NOCOMPRESS  KEEP_DUPLICATES )
 PARTITION BY HASH ("CONCEPT_CD") ;

--
-- Type: INDEX; Owner: I2B2DEMODATA; Name: FACT_NOLOB
--
CREATE INDEX "I2B2DEMODATA"."FACT_NOLOB" ON "I2B2DEMODATA"."OBSERVATION_FACT" ("PATIENT_NUM", "START_DATE", "CONCEPT_CD", "ENCOUNTER_NUM", "INSTANCE_NUM", "NVAL_NUM", "TVAL_CHAR", "VALTYPE_CD", "MODIFIER_CD", "VALUEFLAG_CD", "PROVIDER_ID", "QUANTITY_NUM", "UNITS_CD", "END_DATE", "LOCATION_CD", "CONFIDENCE_NUM", "UPDATE_DATE", "DOWNLOAD_DATE", "IMPORT_DATE", "SOURCESYSTEM_CD", "UPLOAD_ID")
TABLESPACE "I2B2_INDEX" ;

--
-- Type: INDEX; Owner: I2B2DEMODATA; Name: FACT_CNPT_PAT_ENCT_IDX
--
CREATE INDEX "I2B2DEMODATA"."FACT_CNPT_PAT_ENCT_IDX" ON "I2B2DEMODATA"."OBSERVATION_FACT" ("CONCEPT_CD", "INSTANCE_NUM", "PATIENT_NUM", "ENCOUNTER_NUM")
TABLESPACE "I2B2_INDEX" ;

--
-- Type: INDEX; Owner: I2B2DEMODATA; Name: FACT_PATCON_DATE_PRVD_IDX
--
CREATE INDEX "I2B2DEMODATA"."FACT_PATCON_DATE_PRVD_IDX" ON "I2B2DEMODATA"."OBSERVATION_FACT" ("PATIENT_NUM", "CONCEPT_CD", "START_DATE", "END_DATE", "ENCOUNTER_NUM", "INSTANCE_NUM", "PROVIDER_ID", "NVAL_NUM", "VALTYPE_CD")
TABLESPACE "I2B2_INDEX" ;

--
-- Type: INDEX; Owner: I2B2DEMODATA; Name: FACT_CNPT_IDX
--
CREATE INDEX "I2B2DEMODATA"."FACT_CNPT_IDX" ON "I2B2DEMODATA"."OBSERVATION_FACT" ("CONCEPT_CD")
TABLESPACE "I2B2_INDEX" ;

--
-- Type: INDEX; Owner: I2B2DEMODATA; Name: OF_CTX_BLOB
--
CREATE INDEX "I2B2DEMODATA"."OF_CTX_BLOB" ON "I2B2DEMODATA"."OBSERVATION_FACT" ("OBSERVATION_BLOB")
INDEXTYPE IS CTXSYS.CONTEXT PARAMETERS ('SYNC (on commit)');
-- TABLESPACE "I2B2_INDEX" ;

--
-- Type: TRIGGER; Owner: I2B2DEMODATA; Name: TRG_OF_ENCOUNTER_NUM
--
  CREATE OR REPLACE TRIGGER "I2B2DEMODATA"."TRG_OF_ENCOUNTER_NUM"
	before insert on "OBSERVATION_FACT"
	for each row begin
	  if inserting then
		if :NEW."ENCOUNTER_NUM" is null then
		  select SEQ_ENCOUNTER_NUM.nextval into :NEW."ENCOUNTER_NUM" from dual;
		end if;
	  end if;
	end;
/
ALTER TRIGGER "I2B2DEMODATA"."TRG_OF_ENCOUNTER_NUM" ENABLE;
